@@ -22,6 +22,8 @@
 
 #include "ocd_file_import.h"
 
+#include <memory>
+
 #include <QBuffer>
 #include <QDebug>
 #include <QDir>
@@ -50,6 +52,19 @@
 #include "../template_image.h"
 #include "../template_map.h"
 #include "../util.h"
+#include "../util/encoding.h"
+
+
+namespace {
+
+static QTextCodec* codecFromSettings()
+{
+	const auto& settings = Settings::getInstance();
+	const auto name = settings.getSetting(Settings::General_Local8BitEncoding).toByteArray();
+	return Util::codecForName(name);
+}	
+
+} // namespace
 
 
 OcdFileImport::OcdImportedPathObject::~OcdImportedPathObject()
@@ -60,9 +75,13 @@ OcdFileImport::OcdImportedPathObject::~OcdImportedPathObject()
 OcdFileImport::OcdFileImport(QIODevice* stream, Map* map, MapView* view)
  : Importer { stream, map, view }
  , delegate { nullptr }
- , custom_8bit_encoding {  QTextCodec::codecForName("Windows-1252") }
+ , custom_8bit_encoding { codecFromSettings() }
 {
-    // nothing else
+	if (!custom_8bit_encoding)
+	{
+		addWarning(tr("Encoding '%1' is not available. Check the settings."));
+		custom_8bit_encoding = QTextCodec::codecForLocale();
+	}
 }
 
 OcdFileImport::~OcdFileImport()
@@ -71,11 +90,30 @@ OcdFileImport::~OcdFileImport()
 }
 
 
-void OcdFileImport::setCustom8BitEncoding(const char* encoding)
+void OcdFileImport::setCustom8BitEncoding(QTextCodec* encoding)
 {
-	custom_8bit_encoding = QTextCodec::codecForName(encoding);
+	custom_8bit_encoding = encoding;
 }
 
+
+QString OcdFileImport::convertOcdString(const QChar* input, uint maxlen) const
+{
+	auto last = input;
+	if (input)
+	{
+		while (maxlen && *last != 0)
+		{
+			last ++;
+			--maxlen;
+		}
+	}
+	QTextCodec* utf16 = QTextCodec::codecForName("UTF-16LE");
+	Q_ASSERT(utf16);
+	auto decoder = std::unique_ptr<QTextDecoder>(utf16->makeDecoder(QTextCodec::ConvertInvalidToNull));
+	return decoder->toUnicode(reinterpret_cast<const char*>(input), 2*int(last - input));
+}
+
+
 void OcdFileImport::addSymbolWarning(const AreaSymbol* symbol, const QString& warning)
 {
 	addWarning( tr("In area symbol %1 '%2': %3").
@@ -1356,7 +1394,7 @@ LineSymbol* OcdFileImport::importRectangleSymbol(const S& ocd_symbol)
 	symbol->cap_style = LineSymbol::RoundCap;
 	symbol->join_style = LineSymbol::RoundJoin;
 	
-	RectangleInfo rect;
+	auto rect = RectangleInfo();
 	rect.border_line = symbol;
 	rect.corner_radius = 0.001 * convertLength(ocd_symbol.corner_radius);
 	rect.has_grid = ocd_symbol.grid_flags & 1;
@@ -1593,15 +1631,16 @@ Object* OcdFileImport::importObject(const O& ocd_object, MapPart* part, int ocd_
 
 QString OcdFileImport::getObjectText(const Ocd::ObjectV8& ocd_object, int ocd_version) const
 {
+	auto input  = ocd_object.coords + ocd_object.num_items;
+	auto maxlen = uint(sizeof(Ocd::OcdPoint32) * ocd_object.num_text);
 	QString object_text;
 	if (ocd_object.unicode && ocd_version >= 8)
 	{
-		object_text = convertOcdString(reinterpret_cast<const QChar*>(ocd_object.coords + ocd_object.num_items));
+		object_text = convertOcdString(reinterpret_cast<const QChar*>(input), maxlen/2);
 	}
 	else
 	{
-		auto len = uint(qMax(size_t(std::numeric_limits<uint>::max()), sizeof(Ocd::OcdPoint32) * ocd_object.num_text));
-		object_text = convertOcdString<Ocd::Custom8BitEncoding>(reinterpret_cast<const char*>(ocd_object.coords + ocd_object.num_items), len);
+		object_text = convertOcdString<Ocd::Custom8BitEncoding>(reinterpret_cast<const char*>(input), maxlen);
 	}
 	
 	// Remove leading "\r\n"
