@@ -1,5 +1,5 @@
 /*
- *    Copyright 2013, 2014 Kai Pastor
+ *    Copyright 2013, 2014, 2016 Kai Pastor
  *
  *    This file is part of OpenOrienteering.
  *
@@ -17,8 +17,8 @@
  *    along with OpenOrienteering.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef _OPENORIENTEERING_XML_STREAM_UTIL_H_
-#define _OPENORIENTEERING_XML_STREAM_UTIL_H_
+#ifndef OPENORIENTEERING_XML_STREAM_UTIL_H
+#define OPENORIENTEERING_XML_STREAM_UTIL_H
 
 #include <vector>
 
@@ -36,6 +36,61 @@ class MapCoord;
 typedef std::vector<MapCoord> MapCoordVector;
 
 
+/**
+ * This class provides recovery from invalid characters in an XML stream.
+ * 
+ * Some characters are not allowed in well-formed XML 1.0 (cf.
+ * https://www.w3.org/TR/2008/REC-xml-20081126/#NT-Char). While QXmlStreamWriter
+ * will not complain when writing such characters, QXmlStreamReader will raise
+ * a NotWellFormedError. This class will remove offending characters from the
+ * input and reset the stream reader to the state it had when the helper object
+ * was initialized.
+ * 
+ * In a single recovery attempt, the utility tries to handle all offending
+ * characters from the element for wich the tool was constructed. For each
+ * offending character, the whole XML data is parsed again from the start.
+ * That's why multiple corrections may take a long time to run.
+ * 
+ * The XML stream must be based on a QIODevice which supports QIODevice::seek.
+ * 
+ * Synopsis:
+ * 
+ *     XmlRecoveryHelper recovery(xml);
+ *     auto text = xml.readElementText();
+ *     if (xml.hasError() && recovery())
+ *     {
+ *         addWarning(tr("Some invalid characters had to be removed.");
+ *         text = xml.readElementText();
+ *     }
+ */
+class XmlRecoveryHelper
+{
+public:
+	/**
+	 * Constructs a new recovery helper for the given xml stream.
+	 * 
+	 * Captures the current position in the XML stream (QXmlStreamReader::characterOffset()).
+	 */
+	XmlRecoveryHelper(QXmlStreamReader& xml) : xml (xml), recovery_start {xml.characterOffset()} {}
+	
+	/**
+	 * Checks the stream for an error which this utility can handle,
+	 * applies corrections, and resets the stream.
+	 * 
+	 * If this operator returns false if either there was a different type of
+	 * error, or if recovery failed. If it returns true, the stream was modified
+	 * in order to fix the errors which are handled by this utility, and a new
+	 * attempt can be made to parse the remainder of the stream.
+	 */
+	bool operator() ();
+	
+private:
+	QXmlStreamReader& xml;
+	const qint64 recovery_start;
+};
+
+
+
 /**
  * The XmlElementWriter helps to construct a single element in an XML document.
  * 
